#-------------------------------------------------------------------------------
# Name:        MusicPlayer
# Purpose:
#
# Author:      Michael Blackwood
#
# Created:     28/06/2012
# Copyright:   (c) Monxcleyr Productions 2012
# Licence:     GPL v3
#-------------------------------------------------------------------------------
#!/usr/bin/env python

import os, platform
from PySide import QtCore, QtGui
from PySide.phonon import Phonon
from PySide.QtGui import QDialog, QLineEdit, QCheckBox, QFileDialog, QDesktopServices
from PySide.QtCore import QTime

class MusicPlayer():
  def __init__(self, MainWindow, position, instance, attributes):
    self.platform = platform.system()
    #self.platform = 'Linux'
    self.instance = instance
    self.mainWindow = MainWindow
    self.pos = position
    self.setupUi(self.pos)
    if attributes == '':
##      print 'Making new Attributes'
      self.createAttr()
      self.attributes = [self.name, self.filename]
    else:
##      print 'Loading Attributes'
      self.attributes = attributes
      self.loadAttr()
    self.buttons()
    self.directory = QDesktopServices.storageLocation(QDesktopServices.MusicLocation)
    self.media.setTickInterval(1000)
    self.media.tick.connect(self.tick)
    self.lcd.display("00:00")

  def updateDevice(self, device):
    self.audio.setOutputDevice(device)
    Phonon.createPath(self.media, self.audio)

  def kill(self):
    self.filename = ''
    self.name = 'None'
    self.media.stop()
    self.media = Phonon.MediaObject(self.instance)
    self.audio = Phonon.AudioOutput(Phonon.MusicCategory, self.instance)
    self.source = Phonon.MediaSource(self.filename)
    Phonon.createPath(self.media, self.audio)
    self.media.stateChanged.connect(lambda x,y: self.changed(x,y))
    self.label.setText(self.name)

  def loadAttr(self):
    self.name = self.attributes[0]
    self.filename = self.attributes[1]
    self.media = Phonon.MediaObject(self.instance)
    self.audio = Phonon.AudioOutput(Phonon.MusicCategory, self.instance)
    self.source = Phonon.MediaSource(self.filename)
    Phonon.createPath(self.media, self.audio)
    self.label.setText(self.name)
    self.updateUi()

  def createAttr(self):
    self.filename = ''
    self.name = 'None'
    self.media = Phonon.MediaObject(self.instance)
    self.audio = Phonon.AudioOutput(Phonon.MusicCategory, self.instance)
    self.source = Phonon.MediaSource(self.filename)
    Phonon.createPath(self.media, self.audio)
    self.media.stateChanged.connect(lambda x,y: self.changed(x,y))
    self.label.setText(self.name)
    self.updateUi()

  def saveAttr(self):
    self.attributes = [self.name, self.filename]

  def buttons(self):
    self.playButton.clicked.connect(lambda: self.onplaybutton())
    self.resetButton.clicked.connect(lambda: self.onresetbutton())
    self.stopButton.clicked.connect(lambda: self.onstopbutton())
    self.optionButton.clicked.connect(lambda: self.optionDialog())

  def onplaybutton(self):
    state = self.media.state()
    if state == Phonon.State.ErrorState:
      self.name = 'FILE ERROR!!'
      self.label.setText(self.name)
      self.saveAttr()
    elif state != Phonon.State.PausedState and state != Phonon.State.PlayingState:
      self.media.setCurrentSource(self.source)
      self.media.play()
    elif state == Phonon.State.PlayingState:
      self.media.pause()
    elif state == Phonon.State.PausedState:
      time = self.media.currentTime()
      self.media.play()
      self.media.seek(time)
    # reset play button when clicking play on an empty player:
    if self.source.type() == Phonon.MediaSource.Invalid:
      self.playButton.setIcon(QtGui.QIcon(":play.png"))

  def onresetbutton(self):
    self.media.setCurrentSource(self.source)
    self.media.play()
    self.playButton.setIcon(QtGui.QIcon(":pause.png"))
    # reset play button when clicking play on an empty player:
    if self.source.type() == Phonon.MediaSource.Invalid:
      self.playButton.setIcon(QtGui.QIcon(":play.png"))

  def onstopbutton(self):
    self.media.stop()
    self.playButton.setIcon(QtGui.QIcon(":play.png"))

  def clear(self):
    self.media.stop()
    self.filename = ''
    self.source = Phonon.MediaSource(self.filename)
    self.name = 'None'
    self.label.setText(self.name)

  def updateUi(self):
    self.seek = Phonon.SeekSlider(self.media, self.widget)
    # Seems under linux Phonon Doesn't respect which audioOutput instance the
    # Volume slider should control and instead controls the entire program.
    # Volume sliders are disabled in the ubuntu builds of this soundboard until
    # a workaround is discovered.
    if self.platform == 'Linux':
      self.seek.resize(481, 23)
    else:
      self.seek.resize(361, 21)
      self.volume = Phonon.VolumeSlider(self.audio, self.volumeWidget)
      self.volume.resize(111, 21)
      self.volume.show()
    self.seek.show()
    self.playButton.setIcon(QtGui.QIcon(":play.png"))
    self.media.stateChanged.connect(lambda x,y: self.changed(x,y))

  def changed(self, state, oldstate):
    if state == Phonon.State.ErrorState:
      self.error()
    if self.media.remainingTime() == 0:
      self.media.stop()
    if state == Phonon.State.StoppedState:
      self.playButton.setIcon(QtGui.QIcon(":play.png"))
    elif state == Phonon.State.PlayingState:
      self.playButton.setIcon(QtGui.QIcon(":pause.png"))
    elif state == Phonon.State.PausedState:
      self.playButton.setIcon(QtGui.QIcon(":play.png"))

  def saveOptions(self):
    if self.olineEdit.text() == 'Enter file name.':
      self.name = os.path.split(self.filename)[1]
    else:
      self.name = self.olineEdit.text()
    self.label.setText(self.name)
    self.media.stop()
    self.playButton.setIcon(QtGui.QIcon(":play.png"))
    self.source = Phonon.MediaSource(self.filename)
    if self.ocheck.isChecked() == True:
      self.clear()
    self.saveAttr()
    self.instance.savePickle()
    self.odialog.close()

  def optionDialog(self):
    self.odialog = QDialog(parent=self.mainWindow)
    self.odialog.resize(200,100)
    self.odialog.setWindowTitle('Options')
    self.odialog.setModal(True)
    self.odialog.setWindowIcon(QtGui.QIcon(":player.png"))

    # x, y, width, height
    nameButton = QtGui.QPushButton(self.odialog)
    nameButton.setGeometry(QtCore.QRect(30, 70, 60, 23))
    nameButton.setMinimumSize(QtCore.QSize(0, 21))
    nameButton.setText("Save")
    nameButton.setObjectName("nameButton")
    cancelButton = QtGui.QPushButton(self.odialog)
    cancelButton.setGeometry(QtCore.QRect(110, 70, 65, 23))
    cancelButton.setMinimumSize(QtCore.QSize(0, 21))
    cancelButton.setText("Cancel")
    cancelButton.setObjectName("cancelButton")
    cancelButton.clicked.connect(self.odialog.close)
    loadButton = QtGui.QPushButton(self.odialog)
    loadButton.setGeometry(QtCore.QRect(25, 10, 50, 23))
    loadButton.setMinimumSize(QtCore.QSize(0, 21))
    loadButton.setText("Load...")
    loadButton.setObjectName("loadButton")
    self.olineEdit = QLineEdit(self.odialog)
    self.olineEdit.setGeometry(QtCore.QRect(25, 40, 150, 20))
    self.olineEdit.setText('Enter file name.')
    self.ocheck = QCheckBox('Remove File', self.odialog)
    self.ocheck.setGeometry(QtCore.QRect(100, 15, 80, 20))

    loadButton.clicked.connect(self.loadDialog)
    nameButton.clicked.connect(self.saveOptions)
    self.odialog.show()

  def loadDialog(self):
    number = 0
    filename = QFileDialog.getOpenFileName(self.mainWindow, "Open File" , self.directory + '\Sounds', "Sound Files (*.mp3 *.wav);; All Files (*.*)")

    self.filename = str(filename[0])


  def tick(self, time):
    time = QTime(0, (time / 60000) % 60, (time / 1000) % 60)
    self.lcd.display(time.toString('mm:ss'))

  def setupUi(self, position):
    x = position[0]
    y = position[1]
    # 10, 40
    self.line = QtGui.QFrame(self.mainWindow)
    self.line.setGeometry(QtCore.QRect(x, y + 50, 481, 16))
    self.line.setFrameShape(QtGui.QFrame.HLine)
    self.line.setFrameShadow(QtGui.QFrame.Sunken)
    self.label = QtGui.QLabel(self.mainWindow)
    self.label.setGeometry(QtCore.QRect(x + 100, y, 281, 21))
    self.playButton = QtGui.QPushButton(self.mainWindow)
    self.playButton.setGeometry(QtCore.QRect(x, y, 31, 23))
    self.playButton.setMinimumSize(QtCore.QSize(0, 21))
    self.playButton.setText("")
    self.playButton.setIcon(QtGui.QIcon(":play.png"))
    self.resetButton = QtGui.QPushButton(self.mainWindow)
    self.resetButton.setGeometry(QtCore.QRect(x + 60, y, 31, 23))
    self.resetButton.setText("")
    self.resetButton.setObjectName("pauseButton_1")
    self.resetButton.setIcon(QtGui.QIcon(":reset.png"))
    self.optionButton = QtGui.QPushButton(self.mainWindow)
    self.optionButton.setGeometry(QtCore.QRect(x + 460, y, 21, 23))
    self.optionButton.setIcon(QtGui.QIcon(":options.png"))
    self.stopButton = QtGui.QPushButton(self.mainWindow)
    self.stopButton.setGeometry(QtCore.QRect(x + 30, y, 31, 23))
    self.stopButton.setText("")
    self.stopButton.setObjectName("stopButton_1")
    self.stopButton.setIcon(QtGui.QIcon(":stop.png"))
    self.widget = QtGui.QWidget(self.mainWindow)
    if self.platform == 'Linux':
      self.widget.setGeometry(QtCore.QRect(x, y + 30, 481, 21))
    else:
      self.widget.setGeometry(QtCore.QRect(x, y + 30, 361, 21))
      self.volumeWidget = QtGui.QWidget(self.mainWindow)
      self.volumeWidget.setGeometry(QtCore.QRect(x + 370, y + 30, 111, 21))
      self.volumeWidget.setObjectName("volumeWidget_1")
    self.widget.setObjectName("widget_1")
    self.lcd = QtGui.QLCDNumber(self.mainWindow)
    palette = QtGui.QPalette()
    palette.setBrush(QtGui.QPalette.Light, QtCore.Qt.darkGray)
    self.lcd.setPalette(palette)
    self.lcd.setGeometry(QtCore.QRect(x + 390, y, 64, 23))
    self.lcd.setFrameShape(QtGui.QFrame.Box)
    self.lcd.setFrameShadow(QtGui.QFrame.Sunken)
    self.lcd.setLineWidth(1)
    self.lcd.setSmallDecimalPoint(True)
    self.lcd.setSegmentStyle(QtGui.QLCDNumber.Outline)
    self.optionButton.setStatusTip("Options.")

