#-------------------------------------------------------------------------------
# Name:        TabController
# Purpose:
#
# Author:      Michael Blackwood
#
# Created:     28/06/2012
# Copyright:   (c) Monxcleyr Productions 2012
# Licence:     GPL v3
#-------------------------------------------------------------------------------
#!/usr/bin/env python

import soundtab, musictab
from PySide import QtCore, QtGui
from PySide.QtGui import QWidget, QDialog, QLineEdit, QRadioButton

class TabController():

  def __init__(self, mainWindow, instance, parent, attributes):
    self.mainWindow = mainWindow
    self.instance = instance
    self.parent = parent
    if attributes == ['']:
      self.attributesList = [['sound', 0, ['Tab 1!', ['','','','','','','','','','','','','','','','']]], \
      ['music', 1, ['Tab 2!', ['','','','','','','','']]]]
    else:
      self.attributesList = attributes
    self.tablist = []
    self.removeTab = 0
    self.makeFakeTab()

    self.attributesList = sorted(self.attributesList, key = self.sort)
    for attr in self.attributesList:
      if attr[0] == 'sound':
        self.addSoundTab(attr[2])
      if attr[0] == 'music':
        self.addMusicTab(attr[2])

    self.removeFakeTab()
    self.parent.tabCloseRequested.connect(lambda index: self.closeTab(index))

  def sort(self, temp):
    return temp[1]

  def updateDevice(self, device):
    for tab in self.tablist:
      tab.updateDevice(device)

  def getAttributes(self):
    self.attributesList = []
    for i in range(0, len(self.tablist)):
      self.attributesList.append(self.tablist[i].getAttributes())
    return self.attributesList

  def makeFakeTab(self):
    # Creates a blank tab before creating the real tabs, and then deletes it, so there
    # isn't some dumb blank tab. No idea why it does that
    temp = QWidget()
    self.parent.addTab(temp, '')

  def removeFakeTab(self):
    self.parent.widget(0).deleteLater()

  def addNewSoundTab(self, name):
    self.addSoundTab([name, ['','','','','','','','','','','','','','','','']])

  def addSoundTab(self, attr):
    self.tablist.append(soundtab.SoundTab(self.mainWindow, self.instance, self.parent, attr))

  def addNewMusicTab(self, name):
    self.addMusicTab([name, ['','','','','','','','']])

  def addMusicTab(self, attr):
    self.tablist.append(musictab.MusicTab(self.mainWindow, self.instance, self.parent, attr))

  def renameTab(self, name):
    for i in self.tablist:
      if i.widget == self.parent.currentWidget():
        i.renameTab(name)
        self.parent.setTabText(self.parent.currentIndex(), name)

  def closeTab(self, index):
    if self.parent.count() > 1:
      for i in self.tablist:
        if i.widget == self.parent.widget(index):
          self.tablist[self.tablist.index(i)].kill()
          self.tablist.remove(i)
      self.parent.widget(index).deleteLater()

  def kill(self):
    for tab in self.tablist:
      tab.kill()

  def saveTabOptions(self):
    if self.lineEdit.text() == 'New or current tab name!':
      name = 'New Tab!'
    else:
      name = self.lineEdit.text()
    if self.sound.isChecked() == True:
      self.addNewSoundTab(name)
    if self.music.isChecked() == True:
      self.addNewMusicTab(name)
    if self.current.isChecked() == True:
      self.renameTab(name)

    self.dialog.close()
    self.instance.savePickle()

  def tabOptions(self):
    self.dialog = QDialog(parent=self.mainWindow)
    self.dialog.resize(240,120)
    self.dialog.setWindowTitle('Tab Options')
    self.dialog.setModal(True)
    self.dialog.setWindowIcon(QtGui.QIcon(":player.png"))

    # x, y, width, height
    nameButton = QtGui.QPushButton(self.dialog)
    nameButton.setGeometry(QtCore.QRect(30, 90, 60, 23))
    nameButton.setMinimumSize(QtCore.QSize(0, 21))
    nameButton.setText("Save")
    nameButton.setObjectName("nameButton")
    cancelButton = QtGui.QPushButton(self.dialog)
    cancelButton.setGeometry(QtCore.QRect(150, 90, 60, 23))
    cancelButton.setMinimumSize(QtCore.QSize(0, 21))
    cancelButton.setText("Cancel")
    cancelButton.setObjectName("cancelButton")
    cancelButton.clicked.connect(self.dialog.close)
    self.lineEdit = QLineEdit(self.dialog)
    self.lineEdit.setGeometry(QtCore.QRect(25, 62, 190, 20))
    self.lineEdit.setText('New or current tab name!')
    self.lineEdit.setMaxLength(25)
    self.sound = QRadioButton('New Sound Tab', self.dialog)
    self.sound.setGeometry(QtCore.QRect(25, 10, 150, 20))
    self.music = QRadioButton('New Music Tab', self.dialog)
    self.music.setGeometry(QtCore.QRect(25, 25, 150, 20))
    self.current = QRadioButton('Current Tab', self.dialog)
    self.current.setGeometry(QtCore.QRect(25, 40, 150, 20))
    self.current.setChecked(True)

    nameButton.clicked.connect(self.saveTabOptions)
    self.dialog.show()
