#-------------------------------------------------------------------------------
# Name:        Monxcleyr Soundboard
# Purpose:
#
# Author:      Michael Blackwood
#
# Created:     26/6/2012
# Copyright:   (c) Monxcleyr Productions 2012
# Licence:     GPLv3
#-------------------------------------------------------------------------------
#! /usr/bin/env python

#-------------------------------------------------------------------------------
# Monxcleyr Soundboard source requires Python 2.7 and Pyside
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Monxcleyr Soundboard source is built to be both Windows and Linux compatable.
# Any platform specifics such as limitations or user data directories will
# adjust for the system it is running on.
#
# Monxcleyr Soundboard has not been tested on Mac.
#-------------------------------------------------------------------------------

import platform, PySide, sys, os, cPickle, zlib
from files import soundplayer, tabcontroller

from PySide import QtGui
from PySide.QtGui import QApplication, QMainWindow, QTextEdit, QPushButton,\
                         QMessageBox,  QIcon, QFileDialog, QInputDialog, \
                         QDialog, QImage, QDesktopServices, QListWidget, \
                         QListWidgetItem
from PySide.QtCore import QSize, QRect

__version__ = '1.3.2'
from files.ui_soundboard import Ui_MainWindow
import files.grc_soundboard as grc_soundboard

try:
    from PySide.phonon import Phonon
except ImportError:
    app = QtGui.QApplication(sys.argv)
    QtGui.QMessageBox.critical(None, "Monxcleyr Soundboard",
            "Your Qt installation does not have Phonon support.",
            QtGui.QMessageBox.Ok | QtGui.QMessageBox.Default,
            QtGui.QMessageBox.NoButton)
    sys.exit(1)

class MainWindow(QMainWindow, Ui_MainWindow):
  def __init__(self, parent=None):
    super(MainWindow, self).__init__(parent)
    self.setupUi(self)
    self.hasReset = False
    self.audiodevice = ''
    self.mimeTypes = ''


    for mime in Phonon.BackendCapabilities.availableMimeTypes():
      if mime[0:5] == 'audio':
        if mime[6] != 'x':
          self.mimeTypes += str(mime[6:]) + ', '
    self.mimeTypes = self.mimeTypes[:-2]

    # List devices
    self.devicelist = []
    for device in Phonon.BackendCapabilities.availableAudioOutputDevices():
      if device.isValid() == True:
        self.devicelist.append(device)

    # Load save
    if platform.system() == 'Linux':
      dir = QDesktopServices.storageLocation(QDesktopServices.HomeLocation) + '/.config/Monxcleyr Soundboard/'
    else:
      dir = os.path.split(QDesktopServices.storageLocation(QDesktopServices.DataLocation))[0] + '\Monxcleyr\Soundboard\\'
    if not os.path.exists(dir):
      os.makedirs(dir)
    try:
      f = open(os.path.join(dir, 'soundboard.msav'),'rb')
      obj = cPickle.load(f)
      self.savedAttr = cPickle.loads(zlib.decompress(obj))
    except IOError as e:
      # Blank attributes list
      self.savedAttr = ['','','','','','','','',[''], '']


    self.players = []

    for i in range(0, 8):
      self.players.append(soundplayer.SoundPlayer(self.centralwidget, (10, 40 + (i * 70)), self, self.savedAttr[i]))


    self.tabcontroller = tabcontroller.TabController(self.centralwidget, self, self.tabWidget, self.savedAttr[8])

    if self.savedAttr[9] != '':
      self.loadDevice(self.savedAttr[9])

    self.savePickle()


    # logo
    self.label_97.setPixmap(QtGui.QPixmap(":logo.png"))
    self.tabButton.setIcon(QtGui.QIcon(":taboptions.png"))
    self.tabButton.clicked.connect(lambda: self.tabcontroller.tabOptions())
    self.actionAbout.triggered.connect(self.about)
    self.actionHelp.triggered.connect(self.help)
    self.actionReset_Config.triggered.connect(self.reset)
    self.actionDevice.triggered.connect(self.deviceDialog)

    self.tabButton.setStatusTip("Tab Options.")

  def deviceDialog(self):
    self.ddialog = QDialog(parent=self)
    self.ddialog.resize(300,200)
    self.ddialog.setMinimumSize(QSize(300,200))
    self.ddialog.setMaximumSize(QSize(300,200))
    self.ddialog.setWindowTitle('Select Audio Output Device')
    self.ddialog.setModal(True)
    self.ddialog.setWindowIcon(QtGui.QIcon(":player.png"))

    # x, y, width, height
    self.deviceListWidget = QListWidget(self.ddialog)
    self.deviceListWidget.setGeometry(QRect(0, 0, 300, 200))
    self.deviceListWidget.setSortingEnabled(False)
    temp = 0
    for device in self.devicelist:
      QListWidgetItem(self.deviceListWidget)
      self.deviceListWidget.item(temp).setText(device.name())
      temp += 1

    self.deviceListWidget.itemClicked.connect(self.changeDevice)
    self.ddialog.show()

  def loadDevice(self, device):
    if len(self.devicelist) - 1 >= device:
      for i in range(0,8):
        self.players[i].updateDevice(self.devicelist[device])
      self.tabcontroller.updateDevice(self.devicelist[device])


  def changeDevice(self):
    device = self.devicelist[self.deviceListWidget.currentRow()]
    self.audiodevice = device.index()
    for i in range(0,8):
      self.players[i].updateDevice(device)
    self.tabcontroller.updateDevice(device)

    self.ddialog.close()

  def savePickle(self):
    self.statusbar.showMessage('Autosaving...', 2000)
##    print 'Pickle Saved.'
    self.savedAttr = [self.players[0].attributes, self.players[1].attributes, \
      self.players[2].attributes, self.players[3].attributes, self.players[4].attributes, \
      self.players[5].attributes, self.players[6].attributes , self.players[7].attributes]
    self.savedAttr.append(self.tabcontroller.getAttributes())
    self.savedAttr.append(self.audiodevice)

    if platform.system() == 'Linux':
      dir = QDesktopServices.storageLocation(QDesktopServices.HomeLocation) + '/.config/Monxcleyr Soundboard/'
    else:
      dir = os.path.split(QDesktopServices.storageLocation(QDesktopServices.DataLocation))[0] + '\Monxcleyr\Soundboard\\'
    f = open(os.path.join(dir, 'soundboard.msav'),'wb')
    obj = zlib.compress(cPickle.dumps(self.savedAttr,cPickle.HIGHEST_PROTOCOL),9)
    cPickle.dump(obj,f,cPickle.HIGHEST_PROTOCOL)


  def reset(self):
    self.hasReset = True
    self.kill()
    self.savedAttr = ['','','','','','','','',['']]

    for i in range(len(self.tabcontroller.tablist) - 1, -1, -1):
        self.tabWidget.widget(0).deleteLater()
    self.tabWidget.clear()
    del self.tabcontroller.tablist
    self.tabcontroller.tablist = []

    self.tabcontroller = tabcontroller.TabController(self.centralwidget, self, self.tabWidget, self.savedAttr[8])

    self.savePickle()

  def about(self):
    QMessageBox.about(self, "About Monxcleyr Soundboard",
            """<b> Monxcleyr Soundboard</b> v %s
            <p>Copyright &copy; 2012 Monxcleyr Productions, Michael Blackwood.
            <p><a href=http://www.monxcleyr.tk/products/soundboard>www.monxcleyr.tk/products/soundboard</a>
            <p>All rights reserved in accordance with
            GPL v3 or later. No warranties.
            <p>Python %s -  PySide %s - Qt %s on %s""" % \
            (__version__, platform.python_version(), PySide.__version__,\
             PySide.QtCore.__version__, platform.system()))

  def help(self):
    QMessageBox.about(self, "Monxcleyr Soundboard FAQ",
            """<b>Monxcleyr Soundboard FAQ</b>
            <p><b>What audio formats does Monxcleyr Soundboard support?</b>
            <p>Your system should support these audio formats:
            <p>%s
            <p>This depends on your Phonon setup and isn't guaranteed.
            <p><b>Why do tracks fall out of sync with the seek bar?</b>
            <p>Phonon uses bitrates to tell time, VBR files will break this.
            <p><b>How do you pronouce <i>Monxcleyr</i>?
            <p>With much difficulty!
            <p><b>Can I report bugs or ask for further assistance?</b>
            <p>Send an email to contact@monxcleyr.tk to submit bug reports, provide suggestions or get help.
            <p>
            <p><a href=http://www.monxcleyr.tk/products/soundboard>www.monxcleyr.tk/products/soundboard</a>
            <p>""" % \
            (self.mimeTypes))

  def kill(self):
    self.savePickle()
    for i in range(0,8):
      self.players[i].kill()
    self.tabcontroller.kill()


def main():
    app = QApplication(sys.argv)
    app.setApplicationName("Monxcleyr Soundboard")
    frame = MainWindow()
    frame.setWindowIcon(QtGui.QIcon(":player.png"))
    frame.show()
    app.exec_()
    if frame.hasReset == False:
      frame.kill()
if __name__ == '__main__':
    main()
